var previousScrollTop = 0;

function swap(array, index1, index2) {
    const tmp = array[index1];
    array[index1] = array[index2];
    array[index2] = tmp;
}

const ICON_CELL_WIDTH = 25;
const LONG_CELL_WIDTH = 160;
const NAME_CELL_WIDTH = 250;
const DATE_PROPERTIES = ['baselineStartDate', 'baselineEndDate', 'actualStartDate', 'actualEndDate'];
const DATE_FORMAT = {
    MOMENT: 'YYYY-MM-DD',
    SYNCFUSION: 'yyyy-MM-dd'
};
const COLUMN_INDEX = {
    OLD: {
        WBS: 8,
        ACTUAL_START: 9,
        ACTUAL_END: 10,
        ACTUAL_DURATION: 11,
        PROGRESS: 12,
        BASELINE_START_DATE: 13,
        BASELINE_END_DATE: 14
    },
    CURRENT: {
        STATUS: 0,
        MESSAGES: 1,
        WARNINGS: 2,
        CONDITIONS: 3,
        RISKS: 4,
        BUDGET: 5,
        ID: 6,
        NAME: 7,
        OWNER: 8,
        BASELINE_START_DATE: 9,
        BASELINE_END_DATE: 10,
        BASELINE_DURATION: 11,
        PROGRESS: 12,
        ACTUAL_START: 13,
        ACTUAL_END: 14,
        ACTUAL_DURATION: 15
    }
};

$.views.converters({
    'date': val => {
        if (val)
            return moment(String(value)).format('YYYY-MM-DD')
        return '';
    }
});

function createGantt() {
    const projectData = cloneDeep(sampleData);
    removeTime(projectData);
    formatName(projectData);
    const gantt = $("#GanttContainer")
        .ejGantt({
            dataSource: projectData,
            dateFormat: DATE_FORMAT.SYNCFUSION,
            selectionMode: ej.Gantt.SelectionMode.Row,
            allowColumnResize: false,
            allowDragAndDrop: true,
            dragTooltip: {
                showTooltip:true
            },
            taskIdMapping: 'id',
            taskNameMapping: 'name',
            startDateMapping: 'actualStartDate',
            endDateMapping: 'actualEndDate',
            durationMapping: 'actualDuration',
            durationUnit: ej.Gantt.DurationUnit.Day,
            progressMapping: 'progress',
            childMapping: 'children',
            milestoneMapping: 'isMilestone',
            baselineStartDateMapping: 'baselineStartDate',
            baselineEndDateMapping: 'baselineEndDate',
            showTaskNames: false,
            renderBaseline: true,
            treeColumnIndex: 7,
            isResponsive: true,
            allowUnscheduledTask: true,
            enableWBS: true,
            load: function () {
                designColumns(this.getColumns());
            },
            editSettings: {
                allowAdding: true,
                allowEditing: true,
                editMode: 'cellEditing',
                beginEditAction: ej.Gantt.BeginEditAction.Click
            },
            scheduleHeaderSettings: {
                scheduleHeaderType: ej.Gantt.ScheduleHeaderType.Week,
                weekStartDay: 1
            },
            taskbarTooltipTemplate: `<span>{{:#data.item.name}}</span>`,
            enableVirtualization: true,
            taskbarEditing: function (args) {
                args.roundOffDuration = true;
            },
            rowDataBound: args => {
                if (!args.data.item.isStage)
                    $(args.rowElement).find('.e-rowcell').css('font-weight', 'normal');
            },
            stripLines: getStripLines(moment().add(1, "days").toDate, moment().add(-1, "days")),
            create: function(args) {
                this.setScrollTop(previousScrollTop);
            },
            enableContextMenu: true,
            contextMenuOpen: function(args) {
                createContextMenu(args);
            }
        });

    $("#GanttContainer").on('DOMSubtreeModified', () => {
        const $splitter = $('span.e-splitbar', $("#GanttContainer"));
        if ($splitter.length === 0)
            return;

        $("#GanttContainer").off('DOMSubtreeModified');

        // https://www.syncfusion.com/forums/130188/how-to-disable-the-gantt-chart-splitter-to-prevent-moving
        $splitter.removeClass('e-split-divider e-resize');
    });
}

function removeTime(data) {
    data.forEach(node => {
        DATE_PROPERTIES.forEach(property => {
            if (node[property])
                node[property] = moment(node[property])
                    .startOf('day')
                    .format(DATE_FORMAT.MOMENT);

            removeTime(node.children);
        });
    });
}

function getStripLines(startDate, endDate) {
    const today = moment().startOf('day');
    if (moment(startDate).startOf('day').isAfter(today) || moment(endDate).startOf('day').isBefore(today))
        return [];
    return [{
        day: today.add(12, 'hours').format(),
        lineStyle: 'dotted',
        lineColor: 'blue',
        lineWidth: 2
    }];
}

function formatName(data) {
    data.forEach(node => {
        const wbsHtml = `<span class="mr-2" ">${node.wbs}</span>`;
        node.name = wbsHtml + `<span>${node.name}</span>`;
        formatName(node.children);
    });
}

function getColumnsWidth(columns) {
    const width = columns
        .filter(column => column.visible && column.width)
        .reduce((a, b) => a + b.width, 0);
    return `${width}px`;
}

function createContextMenu(args) {
    const item = args.item.item;

    args.contextMenuItems = [];

    args.contextMenuItems.push({
        headerText: 'Add above',
        menuId: 'AddAbove',
        iconClass: 'e-aboveIcon'
    });

    args.contextMenuItems.push({
        headerText: 'task',
        menuId: 'AddAboveMilestone',
        parentMenuId: 'AddAbove',
        eventHandler: function(args) {
            console.log('task');
            this.addRecord(args.data, ej.TreeGrid.RowPosition.Above);
        }
    });

    args.contextMenuItems.push({
        headerText: 'milestone',
        menuId: 'AddAboveMilestone',
        parentMenuId: 'AddAbove',
        eventHandler: function(args) {
            console.log('milestone');
            this.addRecord(args.data, ej.TreeGrid.RowPosition.Above);
        }
    });

    console.log(args.contextMenuItems);
}

function designColumns(columns) {
    columns.splice(COLUMN_INDEX.OLD.WBS, 1);

    columns.splice(COLUMN_INDEX.CURRENT.STATUS, 0, {
        isTemplateColumn: true,
        template: ``,
        width: ICON_CELL_WIDTH,
        headerText: 'status',
        visible: true
    });

    columns.splice(COLUMN_INDEX.CURRENT.MESSAGES, 0, {
        isTemplateColumn: true,
        template: ``,
        width: ICON_CELL_WIDTH,
        headerText: 'messages',
        visible: true
    });

    columns.splice(COLUMN_INDEX.CURRENT.WARNINGS, 0, {
        isTemplateColumn: true,
        template: ``,
        width: ICON_CELL_WIDTH,
        headerText: 'warnings',
        visible: true
    });

    columns.splice(COLUMN_INDEX.CURRENT.CONDITIONS, 0, {
        isTemplateColumn: true,
        template: ``,
        width: ICON_CELL_WIDTH,
        headerText: 'condition',
        visible: true
    });

    columns.splice(COLUMN_INDEX.CURRENT.RISKS, 0, {
        isTemplateColumn: true,
        template: ``,
        width: ICON_CELL_WIDTH,
        headerText: 'risks',
        visible: true
    });

    columns.splice(COLUMN_INDEX.CURRENT.BUDGET, 0, {
        isTemplateColumn: true,
        template: ``,
        width: ICON_CELL_WIDTH,
        headerText: 'budget',
        visible: true
    });

    columns[COLUMN_INDEX.CURRENT.ID].visible = false;

    columns[COLUMN_INDEX.CURRENT.NAME].visible = true;
    columns[COLUMN_INDEX.CURRENT.NAME].width = NAME_CELL_WIDTH;
    columns[COLUMN_INDEX.CURRENT.NAME].headerText = 'taskName';

    columns.splice(COLUMN_INDEX.CURRENT.OWNER, 0, {
        field: 'owner',
        headerText: 'owner',
        visible: true,
        width: LONG_CELL_WIDTH
    });

    columns[COLUMN_INDEX.OLD.ACTUAL_START].headerText = 'actualStart';
    columns[COLUMN_INDEX.OLD.ACTUAL_START].visible = true;
    columns[COLUMN_INDEX.OLD.ACTUAL_START].isTemplateColumn = true;
    columns[COLUMN_INDEX.OLD.ACTUAL_START].showNullText = false;
    columns[COLUMN_INDEX.OLD.ACTUAL_START].template = ``;

    columns[COLUMN_INDEX.OLD.ACTUAL_END].headerText = 'actualEnd';
    columns[COLUMN_INDEX.OLD.ACTUAL_END].visible = true;
    columns[COLUMN_INDEX.OLD.ACTUAL_END].isTemplateColumn = true;
    columns[COLUMN_INDEX.OLD.ACTUAL_END].showNullText = false;
    columns[COLUMN_INDEX.OLD.ACTUAL_END].template = '<span class="text-cell">{{date:#data.item.actualEndDate}}</span>';

    columns[COLUMN_INDEX.OLD.ACTUAL_DURATION].headerText = 'actualDuration';
    columns[COLUMN_INDEX.OLD.ACTUAL_DURATION].visible = true;

    columns[COLUMN_INDEX.OLD.PROGRESS].headerText = 'progress';
    columns[COLUMN_INDEX.OLD.PROGRESS].visible = true;
    columns[COLUMN_INDEX.OLD.PROGRESS].isTemplateColumn = true;
    columns[COLUMN_INDEX.OLD.PROGRESS].template = `}`;

    columns[COLUMN_INDEX.OLD.BASELINE_START_DATE].headerText = 'baselineStartDate';
    columns[COLUMN_INDEX.OLD.BASELINE_START_DATE].visible = true;
    columns[COLUMN_INDEX.OLD.BASELINE_START_DATE].isTemplateColumn = true;
    columns[COLUMN_INDEX.OLD.BASELINE_START_DATE].showNullText = false;
    columns[COLUMN_INDEX.OLD.BASELINE_START_DATE].template = ``;

    columns[COLUMN_INDEX.OLD.BASELINE_END_DATE].headerText = 'baselineEndDate';
    columns[COLUMN_INDEX.OLD.BASELINE_END_DATE].visible = true;
    columns[COLUMN_INDEX.OLD.BASELINE_END_DATE].isTemplateColumn = true;
    columns[COLUMN_INDEX.OLD.BASELINE_END_DATE].showNullText = false;
    columns[COLUMN_INDEX.OLD.BASELINE_END_DATE].template = '<span class="text-cell">{{date:#data.item.baselineEndDate}}</span>';

    columns[COLUMN_INDEX.OLD.ACTUAL_DURATION].isTemplateColumn = true;
    columns[COLUMN_INDEX.OLD.ACTUAL_DURATION].template = ``;

    swap(columns, COLUMN_INDEX.OLD.BASELINE_START_DATE, COLUMN_INDEX.OLD.ACTUAL_START);
    swap(columns, COLUMN_INDEX.OLD.BASELINE_END_DATE, COLUMN_INDEX.OLD.ACTUAL_END);
    columns.push(columns.splice(COLUMN_INDEX.OLD.ACTUAL_DURATION, 1)[0]);

    columns.splice(COLUMN_INDEX.CURRENT.BASELINE_DURATION, 0, {
        isTemplateColumn: true,
        headerText: 'baselineDuration',
        visible: true,
        template: ``,
        width: LONG_CELL_WIDTH
    });
}
